@@ -99,7 +99,7 @@ ManageData <- function(cols, vars, parent=NULL) {
 
     old.fmt <- cols[[idx]]$format
     new.fmt <- as.character(tclvalue(fmt.var))
-    if (is.na(new.fmt))
+    if (new.fmt == "")
       new.fmt <- NULL
     cols[[idx]]$format <<- new.fmt
 
@@ -107,8 +107,6 @@ ManageData <- function(cols, vars, parent=NULL) {
 
     old.fun <- cols[[idx]]$fun
     new.fun <- as.character(tclvalue(tkget(frame2.txt.5.2, '1.0', 'end-1c')))
-    if (new.fun == "")
-      new.fun <- "NA"
     cols[[idx]]$fun <<- new.fun
 
     # Save summary
@@ -129,13 +127,6 @@ ManageData <- function(cols, vars, parent=NULL) {
     if (!is.null(cols[[idx]]$summary))
       cols[[idx]]$class <<- cols[[idx]]$summary$Class
 
-    # Save comments
-
-    new.comments <- sub("\n$", "", tclvalue(tkget(frame4.txt, '1.0', 'end')))
-    if (new.comments == "")
-      new.comments <- NULL
-    cols[[idx]]$comments <<- new.comments
-
     # Save name
 
     SetVarId(idx)
@@ -201,15 +192,6 @@ ManageData <- function(cols, vars, parent=NULL) {
       tkinsert(frame3.txt, "end", sum.str)
 
     tkconfigure(frame3.txt, state="disabled")
-
-    DrawHistogram(cols[[idx]]$summary$Hist)
-
-    # Update comments
-
-    tcl(frame4.txt, "delete", '1.0', 'end')
-    comments <- cols[[idx]]$comments
-    if (!is.null(comments))
-      tkinsert(frame4.txt, "end", comments, sep="")
   }
 
   # Account for change in notebook tab
@@ -229,11 +211,6 @@ ManageData <- function(cols, vars, parent=NULL) {
     if (tabid == frame2$ID) {
       tkfocus(frame2)
 
-    # Arrive at comments tab
-
-    } else if (tabid == frame4$ID) {
-      tkfocus(frame4.txt)
-
     # Arrive at summary tab
 
     } else if (tabid == frame3$ID) {
@@ -251,7 +228,7 @@ ManageData <- function(cols, vars, parent=NULL) {
     tkselection.clear(frame1.lst, 0, n)
     tkselection.set(frame1.lst, n, n)
     tkyview(frame1.lst, n - 1L)
-    cols[[n + 1]] <<- list(name=id, class="logical", fun="NA")
+    cols[[n + 1]] <<- list(name=id, class="logical", fun="")
     UpdateNb()
     SetVarId()
 
@@ -311,8 +288,13 @@ ManageData <- function(cols, vars, parent=NULL) {
     }
     
     new.fun <- EditFunction(cols, index=idx, value.length=n, parent=tt)
-    if (is.null(new.fun))
+    
+    if (is.null(new.fun)) {
+      if (cols[[idx]]$fun == "")
+        DeleteVar()
       return()
+    }
+    
     if (new.fun == "") {
       msg <- paste("Nothing is defined for the function",
                    "and the variable will be deleted.")
@@ -400,50 +382,6 @@ ManageData <- function(cols, vars, parent=NULL) {
     tkyview(frame1.lst, new.idx - 1L)
   }
 
-  # Draw histogram in canvas
-
-  DrawHistogram <- function(h) {
-    tclServiceMode(FALSE)
-    tcl(frame3.cvs, "delete", "all")
-    if (is.null(h))
-      return()
-
-    bars <- lapply(1:length(h$counts),
-                   function(i) list(x=c(h$breaks[i], h$breaks[i + 1]),
-                                    y=c(0, h$counts[i])))
-    xran <- range(h$breaks, na.rm=TRUE)
-    yran <- c(0, max(h$counts, na.rm=TRUE))
-
-    for (bar in bars) {
-      pts <- Xy2mn(bar$x, bar$y, xran, yran)
-      mn <- rep(NA, length(pts$m) * 2)
-      is.odd <- !array(0:1, length(mn))
-      mn[ is.odd] <- pts$m
-      mn[!is.odd] <- pts$n
-      tkcreate(frame3.cvs, "rectangle", .Tcl.args(mn), fill="gray",
-               outline="dark gray", width=1, tag="")
-    }
-    tclServiceMode(TRUE)
-  }
-
-  # Transform coordinates from real to canvas
-
-  Xy2mn <- function(x, y, xran, yran) {
-    m <- w * ((x - xran[1]) / diff(xran))
-    n <- h - (h * ((y - yran[1]) / diff(yran)))
-    list(m=m, n=n)
-  }
-
-  # Scale objects in canvas based on canvas size
-
-  ScaleCanvas <- function() {
-    w0 <- w
-    h0 <- h
-    w <<- as.numeric(tkwinfo("width",  frame3.cvs))
-    h <<- as.numeric(tkwinfo("height", frame3.cvs))
-    tcl(frame3.cvs, "scale", "all", 0, 0, w / w0, h / h0)
-  }
-
   # View data for selected variable
 
   CallViewData <- function() {
@@ -460,6 +398,40 @@ ManageData <- function(cols, vars, parent=NULL) {
     tkconfigure(tt, cursor="arrow")
     tkfocus(tt)
   }
+  
+  # Build historgram
+  
+  CallBuildHistogram <- function(check.variable=FALSE) {
+    SaveNb()
+    idx <- as.integer(tkcurselection(frame1.lst)) + 1L
+    if (length(idx) == 0)
+      return()
+    
+    cont.classes <- c("integer", "numeric")
+    idxs <- which(sapply(cols, function(i) i$class) %in% cont.classes)
+    
+    if (length(idxs) == 0 | (check.variable & !idx %in% idxs)) {
+      msg <- paste("Histogram may only be built for continous variables;",
+                   "that is, variables of class \"numeric\" or \"integer\".")
+      tkmessageBox(icon="info", message=msg, title="Histogram", type="ok", 
+                   parent=tt)
+      return()
+    }
+    
+    ids <- sapply(cols, function(i) ifelse(is.null(i$id), NA, i$id))
+    funs <- sapply(cols, function(i) ifelse(is.null(i$fun), NA, i$fun))
+    d <- sapply(idxs, function(i) EvalFunction(funs[i], cols))
+    
+    var.names <- ids[idxs]
+    if (idx %in% idxs)
+      var.default <- ids[idx]
+    else
+      var.default <- var.names[1]
+    
+    BuildHistogram(as.data.frame(d), var.names=var.names, 
+                   var.default=var.default, parent=tt)
+    tkfocus(tt)
+  }
 
 
   # Main program
@@ -510,7 +482,10 @@ ManageData <- function(cols, vars, parent=NULL) {
         command=SaveNewVar)
   tkadd(menu.edit, "command", label="Delete", command=DeleteVar)
   tkadd(menu.edit, "separator")
-  tkadd(menu.edit, "command", label="View data", command=CallViewData)
+  tkadd(menu.edit, "command", label="Histogram", 
+        command=function() CallBuildHistogram(FALSE))
+  tkadd(menu.edit, "separator")
+  tkadd(menu.edit, "command", label="View data table", command=CallViewData)
 
   menu.arrange <- tkmenu(tt, tearoff=0)
   tkadd(top.menu, "cascade", label="Arrange", menu=menu.arrange, underline=0)
@@ -537,37 +512,40 @@ ManageData <- function(cols, vars, parent=NULL) {
                             command=function() Arrange("front"))
   frame0.but.4 <- ttkbutton(frame0, width=2, image=GetBitmapImage("down"),
                             command=function() Arrange("forward"))
-  frame0.but.5 <- ttkbutton(frame0, width=2, image=GetBitmapImage("plus"),
+  
+  frame0.but.5 <- ttkbutton(frame0, width=2, image=GetBitmapImage("histogram"),
+                            command=function() CallBuildHistogram(TRUE))
+  frame0.but.6 <- ttkbutton(frame0, width=2, image=GetBitmapImage("plus"),
                             command=SaveNewVar)
-  frame0.but.6 <- ttkbutton(frame0, width=2, image=GetBitmapImage("delete"),
+  
+  frame0.but.7 <- ttkbutton(frame0, width=2, image=GetBitmapImage("delete"),
                             command=DeleteVar)
   
-  frame0.but.8  <- ttkbutton(frame0, width=12, text="OK",
-                            command=function() SaveChanges("ok"))
-  frame0.but.9 <- ttkbutton(frame0, width=12, text="Apply",
-                            command=function() SaveChanges("apply"))
-  frame0.but.10 <- ttkbutton(frame0, width=12, text="Cancel",
+  frame0.but.9  <- ttkbutton(frame0, width=12, text="OK",
+                             command=function() SaveChanges("ok"))
+  frame0.but.10 <- ttkbutton(frame0, width=12, text="Apply",
+                             command=function() SaveChanges("apply"))
+  frame0.but.11 <- ttkbutton(frame0, width=12, text="Cancel",
                             command=function() tclvalue(tt.done.var) <- 1)
 
-  frame0.grp.11 <- ttksizegrip(frame0)
+  frame0.grp.12 <- ttksizegrip(frame0)
 
   tkgrid(frame0.but.1, frame0.but.2, frame0.but.3, frame0.but.4, frame0.but.5,
-         frame0.but.6, "x", frame0.but.8, frame0.but.9, frame0.but.10, 
-         frame0.grp.11)
+         frame0.but.6, frame0.but.7, "x", frame0.but.9, frame0.but.10, 
+         frame0.but.11, frame0.grp.12)
 
-  tkgrid.columnconfigure(frame0, 6, weight=1)
+  tkgrid.columnconfigure(frame0, 7, weight=1)
 
   tkgrid.configure(frame0.but.1, frame0.but.2, frame0.but.3, frame0.but.4,
-                   frame0.but.5, frame0.but.6, sticky="n", 
+                   frame0.but.5, frame0.but.6, frame0.but.7, sticky="n", 
                    padx=c(0, 2), pady=c(0, 0))
   tkgrid.configure(frame0.but.1, padx=c(10, 2))
-  tkgrid.configure(frame0.but.5, padx=c(25, 2))
-  tkgrid.configure(frame0.but.8, frame0.but.9, frame0.but.10,
+  tkgrid.configure(frame0.but.9, frame0.but.10, frame0.but.11,
                    padx=c(0, 4), pady=c(15, 10))
-  tkgrid.configure(frame0.but.10, columnspan=2, padx=c(0, 10))
-  tkgrid.configure(frame0.grp.11, sticky="se")
+  tkgrid.configure(frame0.but.11, columnspan=2, padx=c(0, 10))
+  tkgrid.configure(frame0.grp.12, sticky="se")
 
-  tkraise(frame0.but.10, frame0.grp.11)
+  tkraise(frame0.but.11, frame0.grp.12)
 
   tkpack(frame0, fill="x", side="bottom", anchor="e")
 
@@ -604,7 +582,7 @@ ManageData <- function(cols, vars, parent=NULL) {
   tkadd(nb, frame2, text="   Variable   ")
 
   frame2.lab.1.1 <- ttklabel(frame2, text="Name")
-  frame2.lab.2.1 <- ttklabel(frame2, text="Units")
+  frame2.lab.2.1 <- ttklabel(frame2, text="Unit")
   frame2.lab.3.1 <- ttklabel(frame2, text="Format")
   frame2.lab.4.1 <- ttklabel(frame2, text="Class")
   frame2.lab.5.1 <- ttklabel(frame2, text="Function")
@@ -661,40 +639,13 @@ ManageData <- function(cols, vars, parent=NULL) {
 
   tkconfigure(frame3.ysc, command=paste(.Tk.ID(frame3.txt), "yview"))
 
-  frame3.cvs <- tkcanvas(frame3, relief="flat", width=w, height=h,
-                         background="white", confine=TRUE, borderwidth=0,
-                         highlightthickness=0)
-
   tkgrid(frame3.txt, frame3.ysc)
-  tkgrid(frame3.cvs)
 
   tkgrid.configure(frame3.txt, sticky="news", padx=0, pady=0)
   tkgrid.configure(frame3.ysc, sticky="ns", padx=0, pady=0)
-  tkgrid.configure(frame3.cvs, sticky="news", padx=2, pady=2, columnspan=2)
 
   tkgrid.columnconfigure(frame3, 0, weight=1, minsize=25)
   tkgrid.rowconfigure(frame3, 0, weight=1, minsize=25)
-  tkgrid.rowconfigure(frame3, 1, weight=1)
-
-  # Frame 4, comment
-
-  frame4 <- ttkframe(nb, relief="flat", padding=0, borderwidth=0)
-  tkadd(nb, frame4, text="   Comments   ")
-
-  frame4.xsc <- ttkscrollbar(frame4, orient="horizontal")
-  frame4.ysc <- ttkscrollbar(frame4, orient="vertical")
-
-  frame4.txt <- tktext(frame4, bg="white", padx=2, pady=2, width=25, height=8,
-                undo=1, wrap="none", foreground="black", relief="flat",
-                xscrollcommand=function(...) tkset(frame4.xsc,...),
-                yscrollcommand=function(...) tkset(frame4.ysc,...))
-
-  tkconfigure(frame4.xsc, command=paste(.Tk.ID(frame4.txt), "xview"))
-  tkconfigure(frame4.ysc, command=paste(.Tk.ID(frame4.txt), "yview"))
-
-  tkpack(frame4.xsc, side="bottom", fill="x", anchor="w", padx=c(0, 15), pady=0)
-  tkpack(frame4.ysc, side="right",  fill="y", anchor="w", padx=0, pady=0)
-  tkpack(frame4.txt, fill="both", expand=TRUE, pady=0)
 
   # Insert notebook and paned window
 
@@ -710,7 +661,6 @@ ManageData <- function(cols, vars, parent=NULL) {
   tkbind(tt, "<Shift-Control-}>", function() Arrange("front"))
   tkbind(tt, "<Control-[>", function() Arrange("backward"))
   tkbind(tt, "<Shift-Control-{>", function() Arrange("back"))
-  tkbind(tt, "<Configure>", ScaleCanvas)
   tkbind(tt, "<Destroy>", function() tclvalue(tt.done.var) <- 1)
 
   tkbind(nb, "<<NotebookTabChanged>>", ChangeTab)
